
// In test mode you can test the control of the motor.
// It will be turned on for 2 seconds and then turned off for 5 seconds eternally.
#define TEST_MODE  true

// Feed at 9:00 and at 21:00
// You can define one or multiple feeding hours here.
const byte FEED_HOURS[] = { 9, 21 };

// The time in seconds during which the relais turns on the motor
const byte FEED_INTERVAL = 4;

// Relais is connected to pin D2
const byte RELAIS_PIN = 2;

// ATTENTION: integer is used by default for calculations and integer is only 16 bit.
// If these constants would not be uint64_t the calulations would go wrong.
const uint64_t SECOND = 1000;
const uint64_t MINUTE = SECOND * 60;
const uint64_t HOUR   = MINUTE * 60;

// You can adjust the Arduino clock (millis()) by plugging-in the feeder at 12:00.
// When the Arduino is powered up, the tick counter (millis()) starts at zero and the time starts runing at 12 o'clock.
// Stupidly the Arduino Nano board will be reset also when you connect via the Serial monitor in the Arduino IDE by USB connection!
// After a power failure the feeding will be shifted to other hours but it will continue with the same intervals.
// Stupidly the Arduino Nano is reset each time a serial (CDC) connection is established and each time it is closed.
// So it is impossible to set the clock by a command sent from the Serial monitor. 
// (The Teensy boards are more intelligent in this aspect but more expensive.)
// An extra hardware (RTC = Real Time Clock) + battery would be required to always have the corret time in the feeder.
// But it is not important for animals if they are feeded at 9 o'clock or at 12 o'clock.
// Important is that they ARE feeded and the interval is not affected by having the wrong time.
const uint64_t START_TIME = 12 * HOUR;


void setup(void) 
{
    pinMode(RELAIS_PIN, OUTPUT);
    digitalWrite(RELAIS_PIN, LOW);
    
    pinMode(LED_BUILTIN, OUTPUT);
    digitalWrite(LED_BUILTIN, HIGH);
    delay(1000);  // flash LED for 1 second when turning on
    digitalWrite(LED_BUILTIN, LOW);   

    Serial.begin(115200);
}

void loop(void) 
{ 
    #if TEST_MODE
        while (true) // endless loop
        {
            delay(5000);
            digitalWrite(RELAIS_PIN, HIGH);
            delay(2000);
            digitalWrite(RELAIS_PIN, LOW);            
        }
    #endif
  
    static byte u8_LastSecond = 0;  
    static byte u8_LastMinute = 0;
      
    uint64_t u64_Milli = GetMillis64() + START_TIME;
    byte     u8_Second = (u64_Milli / SECOND) % 60;
    byte     u8_Minute = (u64_Milli / MINUTE) % 60;
    byte     u8_Hour   = (u64_Milli / HOUR)   % 24;

    // wait for the full second
    if (u8_LastSecond == u8_Second)
        return;

    u8_LastSecond = u8_Second;

    char s8_Buf[100];
    sprintf(s8_Buf, "Clock: %02d:%02d:%02d", u8_Hour, u8_Minute, u8_Second);
    Serial.println(s8_Buf);

    if (u8_LastMinute == u8_Minute)
        return;

    u8_LastMinute = u8_Minute;

    if (u8_Minute != 0)
        return;

    Serial.println("Hour is full -> check feeding times");

    // check if one of the feeding hours has been reached
    for (int i=0; i<sizeof(FEED_HOURS); i++)
    {
        if (u8_Hour == FEED_HOURS[i])
        {
            Serial.println("Start feeding");
            
            digitalWrite(RELAIS_PIN, HIGH);
            delay(FEED_INTERVAL * SECOND);
            digitalWrite(RELAIS_PIN, LOW);

            Serial.println("Stop feeding");
            break;
        }
    }
}

// We need a special time counter that does not roll over after 49 days (as millis() does) 
uint64_t GetMillis64()
{
    static uint32_t u32_High = 0;
    static uint32_t u32_Last = 0;

    uint32_t u32_Now = millis(); // starts at zero after CPU reset

    // Check for roll-over
    if (u32_Now < u32_Last) u32_High ++;
    u32_Last = u32_Now;

    uint64_t u64_Time = u32_High;
    u64_Time <<= 32;
    u64_Time |= u32_Now;
    return u64_Time;
}


